#include <sys/file.h>
#include <sys/errno.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#define R_OK 04
#define F_OK 00
#define O_RDWR 06

#define maxsize 65536

struct datafilehead
{
    long nblocks;        /* number of blocks in file     */
    long ntraces;        /* number of traces per block   */
    long np;             /* number of elements per trace */
    long ebytes;         /* number of bytes per element  */
    long tbytes;         /* number of bytes per trace    */
    long bbytes;         /* number of bytes per block    */
    short transf;        /* transposed storage flag      */
    short status;        /* status of whole file         */
    long spare1;         /* reserved for future use      */
} main_header;    

struct datablockhead
{
    short scale;         /* scaling factor               */
    short status;        /* status of data in block      */
    short index;         /* block index                  */
    short spare3;        /* reserved for future use      */
    long  ctcount;       /* completed transients in fids */
    float lpval;         /* left phase in phasefile      */
    float rpval;         /* right phase in phasefile     */
    float lvl;           /* level drift correction       */
    float tlt;           /* tilt drift correction        */
} block_header;


void getfid(int *si, int *nproj, float *wbuff, char infile_name1[]);
void putfid(int *si, float *wbuff, char infile_name1[]);

main(int argc, char *argv[])
{
  float    wrkbuff0[maxsize],wrkbuff1[maxsize];
  char 	fil_in[100];  
  int j, ne, skip, size, nf, maxproj;

  strcpy(fil_in,argv[1]);
  getfid(&size,&maxproj,&wrkbuff0[0],fil_in);
  skip=atoi(argv[2]);
  fprintf(stderr,"shuffling every %d fid\n", maxproj/skip);
  for (ne=0;ne<maxproj/skip;ne++)
  {
    for (nf=0;nf<skip;nf++)
    {
      for (j=0;j<size;j++)
      {
        wrkbuff1[(nf+ne*skip)*size+j] = wrkbuff0[(nf*maxproj/skip+ne)*size+j];
      }
    }
  }
  putfid(&size,&wrkbuff1[0],fil_in);
}
void getfid(int *si, int *nproj, float *wbuff, char infile_name1[])
    
{
    long    element_size, maxval=16384, scale_factor, temp;
    int     i, j, k,l, nfids_in, in_file, np;
    float   stor, max;
    FILE    *in_ptr;
    char    string[100];
    char    *malloc();
    union   precision {
		short  *sh;
		long   *lo;
	    } *in_data;


    
    /****
    * Determine the accessibility of the input directory.    ****/

    if ((access(infile_name1, F_OK | R_OK))) 
    {
	perror("Access");
	printf("Input file either does not exist, or cannot be read.\n");
	exit(2);
    }

    /****
    * Open input fid file.
    ****/
    strcpy(string, infile_name1);
    in_file = open(string, O_RDWR);
    if (in_file < 0) 
    {
	perror("Open");
	printf("Can't open input fid file.\n");
	exit(4);
    }

    /****
    * Read main_header at top of input file.
    ****/
    if (read(in_file, &main_header, 32) != 32) 
    {
	perror ("read");
	exit(5);
    }
    np = main_header.np;
    *si=np;
    element_size = main_header.ebytes;
    nfids_in = main_header.nblocks;
    *nproj=nfids_in;
 
    /****
    * Allocate space for input data.  All calculations will be done
    * with "long" integers (normal "int" on the SUN), so wee will
    * allocate space for this, even if the data is "short".
    ****/
    in_data = (union precision *)malloc(nfids_in*sizeof(int*));
    for (i=0; i<nfids_in; i++) 
    {
	in_data[i].lo = (long *)malloc(np*sizeof(long));    
    }
    /****
    * Read in raw data.
    ****/
    for (i=0; i<nfids_in; i++) 
    {
    if (read(in_file, &block_header, 28) != 28) 
    {
	perror ("read2");
	exit(5);
    }
    if (read(in_file, in_data[i].lo, np*sizeof(float)) != np*sizeof(float)) 
    {
	perror ("read3");
	exit(5);
    }
    }
    for (i=0; i<nfids_in; i++)
      {
      for (j=0; j<np; j++)
      {
       wbuff[i*np+j]=(float)in_data[i].lo[j];
      }
      }
    fprintf(stderr,"done reading .. ");
    fprintf(stderr,"%5d points in %3d projections (%2d el. size)\n",*si,nfids_in,element_size);
    close(in_file);
    free(in_data);
  }/*getdata*/


void putfid(int *si, float *wbuff, char infile_name1[])
    
{
    long    element_size, maxval=16384, scale_factor, temp;
    int     i, j, k,l, nfids_out, out_file, np;
    float   stor, max;
    FILE    *out_ptr;
    char    string[100];
    char    *malloc();
    union   precision {
		short  *sh;
		long   *lo;
	    } *out_data;

    strcpy(string, infile_name1);
    out_file = open(string, O_RDWR);
    if (out_file < 0) 
    {
	perror("Open");
	printf("Can't open output fid file.\n");
	exit(4);
    }
    np = main_header.np;
    nfids_out = main_header.nblocks;
 
    /****
    * Allocate space for output data.  All calculations will be done
    * with "long" integers (normal "int" on the SUN), so wee will
    * allocate space for this, even if the data is "short".
    ****/
    out_data = (union precision *)malloc(nfids_out*sizeof(int*));
    for (i=0; i<nfids_out; i++) 
    {
	out_data[i].lo = (long *)malloc(np*sizeof(long));    
    }
    write(out_file, &main_header, 32);

    for (i=0; i<nfids_out; i++)
      {
      for (j=0; j<np/2-1; j++)
      {
	out_data[i].lo[2*j] = wbuff[i*np+2*j];
	out_data[i].lo[2*j+1] = wbuff[i*np+2*j+1];
      }
    }
    for (k=0; k<nfids_out; k++) {
	block_header.index = k + 1;
	write(out_file, &block_header, 28);
	write(out_file, out_data[k].lo, np*sizeof(float));
    }
    close(out_file);
    free(out_data);
}
